#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ncurses.h>

#include "ui.h"
#include "othello.h"
#include "debug.h"

int main() {
    int min_y = 26 + 6, min_x = 42 + 14 + 15 + 4;
    int row = 0, col = 0;
    unsigned int round = 0;
    unsigned int player = player_one; /* first player is black */
    bool exit_condition = false;
    unsigned int nb_white = 0, nb_black = 0;

    const char* title_msg = "Jeu Othello";
    const char* score_msg = "Pions %s: %d";
    const char* invalid_move_msg = "Coup invalide";
    const char* player_msg = "Joueur %d (%s) joue !";
    const char* winner_msg = "Joueur %d (%s) gagne !";
    const char* draw_msg = "Egalite !";
    const char* exit_msg = "Pressez une touche pour sortir ou \'r\' pour rejouer";

    /* linked list of playable shots */
    struct shots_list_s playable_shots;
    INIT_LIST_HEAD(&playable_shots.list);

    unsigned int pawns[board_size][board_size] = {
        {0, 0},
        {0, 0}
    };
    init_pawns(pawns);

    initscr();
    if (has_colors() == false) {
        endwin();
        printf("Votre terminal ne supporte pas les couleurs.\n");
        exit(EXIT_FAILURE);
    }
    start_color();
    getmaxyx(stdscr, row, col);
    if (row < min_y || col < min_x) {
        endwin();
        printf("Votre terminal est trop petit pour afficher ce jeu.\n");
        printf("Merci d'agrandir la fenetre de votre terminal.\n");
        exit(EXIT_FAILURE);
    }
    echo();
    curs_set(0);

    /* center */
    int center_y = row/2;
    int center_x = col/2;
    /* base coordinates to center the board */
    int board_center_y = center_y - 26/2;
    int board_center_x = center_x - 42/2;

    do {
        print_board(board_center_y, board_center_x);
        print_pawns(board_center_y, board_center_x, pawns);

        mvprintw(center_y - 26/2 - 4, (center_x - strlen(title_msg)/2), title_msg);

        player = current_player(round);

        if (player == player_one) {
            mvprintw(center_y - 26/2 - 2, center_x - snprintf(NULL, 0, player_msg, player, "noir")/2, player_msg, player, "noir");
        } else {
            mvprintw(center_y - 26/2 - 2, center_x - snprintf(NULL, 0, player_msg, player, "blanc")/2, player_msg, player, "blanc");
        }

        nb_white = count_pawns_type(pawns, white);
        nb_black = count_pawns_type(pawns, black);

        mvprintw(center_y, center_x - 42/2 - snprintf(NULL, 0, score_msg, "noirs", nb_black) - 2, score_msg, "noirs", nb_black);
        mvprintw(center_y, center_x + 42/2 + 2, score_msg, "blancs", nb_white);

        build_playable_shots_list(player, &playable_shots, pawns);
        print_shots_list(board_center_y, board_center_x, &playable_shots);

        display_array(1, 1, pawns);

        int y;
        char x_char;
        bool input_ok = false;
        bool first_run = true;
        unsigned int nb_pawns_reversed = 0;
        do {
            y = 0;
            x_char = (char)"";
            const char* prompt_msg = "Prochain pion ? (ligne colonne - chiffre lettre):";
            int prmt_rt = prompt_values(stdscr, center_y + 26/2 + 1, (center_x - strlen(prompt_msg)/2), prompt_msg, &y, &x_char);
            int x = map_col_letter_to_index(x_char);
            /* TODO: a comparaison to the linked list of playable shots is better */
            if (is_legal_shot(y, x, player, pawns) && prmt_rt == 1) {
                nb_pawns_reversed = valid_shot(y, x, player, pawns);
                input_ok = true;
                clear();
            } else {
                mvprintw(center_y + 26/2 + 4, (center_x - strlen(invalid_move_msg)/2), invalid_move_msg);
            }
        } while (!input_ok);

        free_shots_list(&playable_shots);

        round++; /* increment the round count */

        /* here are all the end of the game conditions */
        if (is_board_full(pawns)) {
            print_board(board_center_y, board_center_x);
            /* print the updated pawns array before exiting */
            print_pawns(board_center_y, board_center_x, pawns);
            unsigned int winner = eval_winner(nb_white, nb_black);
            if (winner != 0) {
                if (winner == player_one) {
                    mvprintw(center_y - 26/2 - 2, center_x - snprintf(NULL, 0, winner_msg, winner, "noir")/2, winner_msg, winner, "noir");
                } else {
                    mvprintw(center_y - 26/2 - 2, center_x - snprintf(NULL, 0, winner_msg, winner, "blanc")/2, winner_msg, winner, "blanc");
                }
            } else {
                mvprintw(center_y - 26/2 - 2, (center_x - strlen(draw_msg)/2), draw_msg);
            }
            mvprintw(center_y + 26/2 + 1, (center_x - strlen(exit_msg)/2), exit_msg);
            exit_condition = true;
            /*  getch() is blocking */
            int key_exit = getch();
            if (key_exit == 'r') {
                round = 0;
                player = player_one;
                nb_white = nb_black = 0;
                /* FIXME: do not seem properly reset the pawns 2D array */
                init_pawns(pawns);
                exit_condition = false;
                clear();
            }
        }

        refresh();

    } while (!exit_condition);

    endwin();

    exit(EXIT_SUCCESS);
}
